package org.eclipse.jdt.internal.compiler.lookup;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeVariableBinding;

public class InferenceContext {
  private TypeBinding[][][] collectedSubstitutes;
  
  MethodBinding genericMethod;
  
  int depth;
  
  int status;
  
  TypeBinding expectedType;
  
  boolean hasExplicitExpectedType;
  
  public boolean isUnchecked;
  
  TypeBinding[] substitutes;
  
  static final int FAILED = 1;
  
  public InferenceContext(MethodBinding genericMethod) {
    this.genericMethod = genericMethod;
    TypeVariableBinding[] typeVariables = genericMethod.typeVariables;
    int varLength = typeVariables.length;
    this.collectedSubstitutes = new TypeBinding[varLength][3][];
    this.substitutes = new TypeBinding[varLength];
  }
  
  public TypeBinding[] getSubstitutes(TypeVariableBinding typeVariable, int constraint) {
    return this.collectedSubstitutes[typeVariable.rank][constraint];
  }
  
  public boolean hasUnresolvedTypeArgument() {
    for (int i = 0, varLength = this.substitutes.length; i < varLength; i++) {
      if (this.substitutes[i] == null)
        return true; 
    } 
    return false;
  }
  
  public void recordSubstitute(TypeVariableBinding typeVariable, TypeBinding actualType, int constraint) {
    int length;
    TypeBinding[][] variableSubstitutes = this.collectedSubstitutes[typeVariable.rank];
    TypeBinding[] constraintSubstitutes = variableSubstitutes[constraint];
    if (constraintSubstitutes == null) {
      length = 0;
      constraintSubstitutes = new TypeBinding[1];
    } else {
      length = constraintSubstitutes.length;
      for (int i = 0; i < length; i++) {
        TypeBinding substitute = constraintSubstitutes[i];
        if (substitute == actualType)
          return; 
        if (substitute == null) {
          constraintSubstitutes[i] = actualType;
          // Byte code: goto -> 114
        } 
      } 
      System.arraycopy(constraintSubstitutes, 0, constraintSubstitutes = new TypeBinding[length + 1], 0, length);
    } 
    constraintSubstitutes[length] = actualType;
    variableSubstitutes[constraint] = constraintSubstitutes;
  }
  
  public String toString() {
    StringBuffer buffer = new StringBuffer(20);
    buffer.append("InferenceContex for ");
    int i, length;
    for (i = 0, length = this.genericMethod.typeVariables.length; i < length; i++)
      buffer.append(this.genericMethod.typeVariables[i]); 
    buffer.append(this.genericMethod);
    buffer.append("\n\t[status=");
    switch (this.status) {
      case 0:
        buffer.append("ok]");
        break;
      case 1:
        buffer.append("failed]");
        break;
    } 
    if (this.expectedType == null) {
      buffer.append(" [expectedType=null]");
    } else {
      buffer.append(" [expectedType=").append(this.expectedType.shortReadableName()).append(']');
    } 
    buffer.append(" [depth=").append(this.depth).append(']');
    buffer.append("\n\t[collected={");
    for (i = 0, length = (this.collectedSubstitutes == null) ? 0 : this.collectedSubstitutes.length; i < length; i++) {
      TypeBinding[][] collected = this.collectedSubstitutes[i];
      for (int m = 0; m <= 2; m++) {
        TypeBinding[] constraintCollected = collected[m];
        if (constraintCollected != null)
          for (int n = 0, clength = constraintCollected.length; n < clength; n++) {
            buffer.append("\n\t\t").append((this.genericMethod.typeVariables[i]).sourceName);
            switch (m) {
              case 0:
                buffer.append("=");
                break;
              case 1:
                buffer.append("<:");
                break;
              case 2:
                buffer.append(">:");
                break;
            } 
            if (constraintCollected[n] != null)
              buffer.append(constraintCollected[n].shortReadableName()); 
          }  
      } 
    } 
    buffer.append("}]");
    buffer.append("\n\t[inferred=");
    int count = 0;
    for (int j = 0, k = (this.substitutes == null) ? 0 : this.substitutes.length; j < k; j++) {
      if (this.substitutes[j] != null) {
        count++;
        buffer.append('{').append((this.genericMethod.typeVariables[j]).sourceName);
        buffer.append("=").append(this.substitutes[j].shortReadableName()).append('}');
      } 
    } 
    if (count == 0)
      buffer.append("{}"); 
    buffer.append(']');
    return buffer.toString();
  }
}
